%global module Ipopt

# Define a macro for calling ../configure instead of ./configure
%global dconfigure %(printf %%s '%configure' | sed 's!\./configure!../configure!g')

## Define libraries' destination
%global _incmpidir %{_includedir}/openmpi-%{_arch}
%global _libmpidir %{_libdir}/openmpi/lib

## Define if use openmpi or not
%global with_openmpi 1

%if 0%{?fedora} > 21
%global with_asl 1
%else
%global with_asl 0
%endif

Name:           coin-or-%{module}
Group:          Applications/Engineering
Summary:        Interior Point OPTimizer
Version:        3.12.4
Release:        3%{?dist}
License:        EPL and GPLv2+
URL:            https://projects.coin-or.org/%{module}
Source0:        http://www.coin-or.org/download/pkgsource/%{module}/%{module}-%{version}.tgz
BuildRequires:  atlas-devel
BuildRequires:  blas-devel
BuildRequires:  gcc-gfortran
BuildRequires:  doxygen
BuildRequires:  glpk-devel
BuildRequires:  graphviz
BuildRequires:  lapack-devel
BuildRequires:  MUMPS-devel >= 4.10.0-21
%if %{with_asl}
BuildRequires:	mp-devel
%endif
BuildRequires:  openssh-clients
BuildRequires:  pkgconfig
BuildRequires:  readline-devel

Requires:       %{name}-common = %{version}-%{release}

# Build with parallel mumps solver
Patch0:	%{name}-mumps.patch

# Correct underlink of libdl
Patch1:	%{name}-underlink.patch


%description
Ipopt (Interior Point OPTimizer, pronounced eye-pea-Opt) is a software
package for large-scale nonlinear optimization. It is designed to find
(local) solutions of mathematical optimization problems of the from

   min     f(x)
x in R^n

s.t.       g_L <= g(x) <= g_U
           x_L <=  x   <= x_U

where f(x): R^n --> R is the objective function, and g(x): R^n --> R^m are
the constraint functions. The vectors g_L and g_U denote the lower and upper
bounds on the constraints, and the vectors x_L and x_U are the bounds on
the variables x. The functions f(x) and g(x) can be non-linear and non-convex,
but should be twice continuously differentiable. Note that equality
constraints can be formulated in the above formulation by setting the
corresponding components of g_L and g_U to the same value.

%package	devel
Summary:	Development files for %{name}
Group:		Development/Libraries
Requires:	coin-or-CoinUtils-devel
Requires:	MUMPS-devel >= 4.10.0-21
Requires:	%{name}%{?_isa} = %{version}-%{release}

%description    devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}.

%package	common
Summary:	Documentation files for %{name}
BuildArch:	noarch
Provides:       %{name}-common = 0:%{version}-%{release}
Obsoletes:      %{name}-doc <= 0:3.11.9-3  

%description	common
This package contains the HTML documentation,
a PDF tutorial to use %{name} and related
license files.

########################################################
%if 0%{?with_openmpi}
%package openmpi
Summary: %{name} compiled against openmpi
BuildRequires:	MUMPS-openmpi-devel >= 4.10.0-21
BuildRequires:	blacs-openmpi-devel
BuildRequires:	openmpi-devel
BuildRequires:	scalapack-openmpi-devel

Requires: %{name}-common = %{version}-%{release}
Requires: openmpi
%description openmpi
%{name} libraries compiled against openmpi

%package openmpi-devel
Summary: The %{name} headers and development-related files
Requires: %{name}-openmpi%{?_isa} = %{version}-%{release}
%description openmpi-devel
Shared links, header files for %{name}.
%endif
##########################################################

%prep
%setup -q -n %{module}-%{version}
%patch1 -p0

# Update the various config.guess to upstream release for aarch64 support
for cdir in `find ./ -name config.guess`
do
       cp /usr/lib/rpm/config.{guess,sub} `dirname $cdir`
done

%build

#######################################################
## Build serial version

mkdir -p %{name}-%{version}-serial
cd %{name}-%{version}-serial
CFLAGS="%{optflags} -DIPOPT_BUILD -DHAVE_MPI_INITIALIZED=0 -I%{_includedir}/MUMPS"
LDFLAGS="-L%{_libdir} -ldmumps -lblas -llapack"
CXXFLAGS="%{optflags} -DIPOPT_BUILD -DHAVE_MPI_INITIALIZED=0 -I%{_includedir}/MUMPS"
CXXLIBS="-L%{_libdir} -ldmumps -lblas -llapack"
%dconfigure CC=gcc CXX=g++ F77=gfortran CFLAGS="$CFLAGS" CXXFLAGS="$CXXFLAGS" \
            LDFLAGS="$LDFLAGS" CXXLIBS="$CXXLIBS" \
 --with-mumps-incdir=%{_includedir}/MUMPS --with-mumps-lib="-L%{_libdir} -ldmumps" \
 --with-blas-incdir=%{_includedir}/lapacke --with-blas-lib="-L%{_libdir} -lblas" \
 --with-lapack-incdir=%{_includedir}/lapacke --with-lapack-lib="-L%{_libdir} -llapack" \
%if %{with_asl}
 --with-asl-lib="-lasl -lmp" --with-asl-incdir="%{_includedir}/asl" \
%endif
 --enable-shared

sed -i 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g' libtool
sed -i 's|^runpath_var=LD_RUN_PATH|runpath_var=DIE_RPATH_DIE|g' libtool

echo 'LATEX_BATCHMODE = YES' >> doxydoc/doxygen.conf

export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:%{_libdir}
make %{?_smp_mflags} all doxydoc
cd ..


#######################################################

#######################################################
## Build MPI version
%if 0%{?with_openmpi}
patch -p0 < %{PATCH0}

mkdir -p %{name}-%{version}-openmpi
cd %{name}-%{version}-openmpi
%{_openmpi_load}
CFLAGS="%{optflags} -DHAVE_MPI_INITIALIZED=1 -I%{_includedir}/openmpi-%{_arch} -DIPOPT_BUILD"
LDFLAGS="-L%{_libdir}/openmpi/lib -lmpi -lmpi_cxx -lmpi_mpifh -lmpiblacs -lscalapack -ldmumps"
CXXFLAGS="%{optflags} -DHAVE_MPI_INITIALIZED=1 -I%{_includedir}/openmpi-%{_arch} -DIPOPT_BUILD"
CXXLIBS="-L%{_libdir}/openmpi/lib -lmpi -lmpi_cxx -lmpi_mpifh -lmpiblacs -lscalapack -ldmumps" 
%dconfigure --with-mumps-incdir=%{_includedir}/openmpi-%{_arch} --with-mumps-lib="-L%{_libdir}/openmpi/lib -ldmumps" \
%if %{with_asl}
            --with-asl-lib="-lasl -lmp" --with-asl-incdir="%{_includedir}/asl" \
%endif
            MPICC=%{_libdir}/openmpi/bin/mpicc \
            MPICXX=%{_libdir}/openmpi/bin/mpic++ \
            MPIF77=%{_libdir}/openmpi/bin/mpifort \
            ADD_CFLAGS="-fopenmp" \
            ADD_FFLAGS="-fopenmp" \
            ADD_CXXFLAGS="-fopenmp" \ 
            CFLAGS="$CFLAGS" CXXFLAGS="$CXXFLAGS" \
            LDFLAGS="$LDFLAGS" CXXLIBS="$CXXLIBS" 
sed -i 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g' libtool
sed -i 's|^runpath_var=LD_RUN_PATH|runpath_var=DIE_RPATH_DIE|g' libtool
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:%{_libdir}/openmpi/lib 
make %{?_smp_mflags} all
%{_openmpi_unload}
cd ..
%endif

%clean
rm -rf $RPM_BUILD_ROOT

%install

#######################################################
## Install MPI version
%if 0%{?with_openmpi}

mkdir -p $RPM_BUILD_ROOT%{_libmpidir}
mkdir -p $RPM_BUILD_ROOT%{_incmpidir}
mkdir -p %{name}-%{version}-openmpi/headers

cp -a %{name}-%{version}-openmpi/Ipopt/src/Interfaces/.libs/libipopt.so* \
       $RPM_BUILD_ROOT%{_libmpidir}
%if %{with_asl}
cp -a %{name}-%{version}-openmpi/Ipopt/src/Apps/AmplSolver/.libs/libipoptamplinterface.so* \
       $RPM_BUILD_ROOT%{_libmpidir}
%endif

cd %{name}-%{version}-openmpi
 for i in `find ./.* -type f \( -name "*.h" -o -name "*.hpp" \)`; do
 cp -p $i headers
done
cd ..

##Copy MPI header files
install -p -m 644 %{name}-%{version}-openmpi/headers/* $RPM_BUILD_ROOT%{_incmpidir}

%endif

#######################################################
## Install serial version

mkdir -p $RPM_BUILD_ROOT%{_libdir}/pkgconfig
mkdir -p $RPM_BUILD_ROOT%{_includedir}/coin
mkdir -p $RPM_BUILD_ROOT%{_docdir}/%{name}

mkdir -p %{name}-%{version}-serial/headers

##Copy libraries
cp -a %{name}-%{version}-serial/Ipopt/src/Interfaces/.libs/libipopt.so* \
       $RPM_BUILD_ROOT%{_libdir}
install -p -m 644 %{name}-%{version}-serial/Ipopt/ipopt.pc $RPM_BUILD_ROOT%{_libdir}/pkgconfig
%if %{with_asl}
cp -a %{name}-%{version}-serial/Ipopt/src/Apps/AmplSolver/.libs/libipoptamplinterface.so* \
       $RPM_BUILD_ROOT%{_libdir}
%endif

cd %{name}-%{version}-serial
 for i in `find ./.* -type f \( -name "*.h" -o -name "*.hpp" \)`; do
 cp -p $i headers
done
cd ..

##Copy header and documentation files
install -p -m 644 %{name}-%{version}-serial/headers/* $RPM_BUILD_ROOT%{_includedir}/coin
cp -far %{name}-%{version}-serial/doxydoc/html $RPM_BUILD_ROOT%{_docdir}/%{name}
cp -p Ipopt/doc/documentation.pdf $RPM_BUILD_ROOT%{_docdir}/%{name}/Ipopt_Tutorial.pdf

#######################################################

%check

cd %{name}-%{version}-serial
LD_LIBRARY_PATH=$RPM_BUILD_ROOT%{_libdir}:$LD_LIBRARY_PATH make test
cd ..

%if 0%{?with_openmpi}
%{_openmpi_load}
cd %{name}-%{version}-openmpi
LD_LIBRARY_PATH=$RPM_BUILD_ROOT%{_libmpidir}:$LD_LIBRARY_PATH make test
cd ..
%{_openmpi_unload}
%endif

%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig

%files
%{_libdir}/*.so.*

%files	devel
%{_includedir}/coin/*
%{_libdir}/*.so
%{_libdir}/pkgconfig/*.pc

#######################################################
## Install MPI version
%if 0%{?with_openmpi}

%files openmpi
%{_libmpidir}/*.so.*

%files openmpi-devel
%{_libmpidir}/*.so
%{_incmpidir}/*

%endif

%files common
%doc Ipopt/AUTHORS Ipopt/LICENSE ChangeLog
%license LICENSE
%dir %{_docdir}/%{name}
%{_docdir}/%{name}/html
%{_docdir}/%{name}/*.pdf

%changelog
* Sun Aug 16 2015 Zbigniew Jędrzejewski-Szmek <zbyszek@in.waw.pl> - 3.12.4-3
- Rebuild for MPI provides

* Mon Aug 10 2015 Sandro Mani <manisandro@gmail.com> - 3.12.4-2
- Rebuild for RPM MPI Requires Provides Change

* Mon Aug 10 2015 Antonio Trande <sagitterATfedoraproject.org> - 3.12.4-1
- Update to 3.12.4

* Sat Jun 20 2015 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.12.2-3
- Full rebuild of coin-or stack.

* Wed Jun 17 2015 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.12.3-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_23_Mass_Rebuild

* Sat Apr 18 2015 Fedora Release Monitoring <release-monitoring@fedoraproject.org> - 3.12.3-1
- Update to 3.12.3 (#1213024)

* Sun Apr 05 2015 Antonio Trande <sagitterATfedoraproject.org> - 3.12.2-1
- Update to 3.12.2

* Mon Mar 09 2015 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.12.1-5
- Rebuild (MUMPS broken dependencies on F22)

* Sun Feb 22 2015 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.12.1-4
- Add conditional to enable the asl solver

* Sun Feb 22 2015 Antonio Trande <sagitterATfedoraproject.org> - 3.12.1-3
- Rebuild again (broken dependencies)

* Sat Feb 21 2015 Antonio Trande <sagitterATfedoraproject.org> - 3.12.1-2
- Rebuild after MUMPS update

* Tue Feb 17 2015 Antonio Trande <sagitterATfedoraproject.org> - 3.12.1-1
- Update to 3.12.1

* Wed Feb 04 2015 Marcin Juszkiewicz <mjuszkiewicz@redhat.com> - 3.12.0-2
- Correct aarch64 build again (#1185848)

* Tue Jan 27 2015 Antonio Trande <sagitterATfedoraproject.org> - 3.12.0-1
- Update to 3.12.0 (bz#1152812)

* Mon Jan 26 2015 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.11.10-3
- Correct aarch64 build (#1185848)

* Fri Jan 23 2015 Antonio Trande <sagitterATfedoraproject.org> - 3.11.10-2
- Fix libraries's symlinks (bz#1152812)

* Mon Jan 19 2015 Antonio Trande <sagitterATfedoraproject.org> - 3.11.10-1
- Update to 3.11.10

* Sun Oct 26 2014 Peter Robinson <pbrobinson@fedoraproject.org> - 3.11.9-4
- Update config.guess config.sub for new arch (aarch64/ppc64le) support

* Sat Oct 04 2014 Antonio Trande <sagitterATfedoraproject.org> - 3.11.9-3
- Obsolete coin-or-Ipopt-doc sub-package

* Tue Sep 09 2014 Antonio Trande <sagitterATfedoraproject.org> - 3.11.9-2
- Performed conditional openmpi build macro
- Built libraries against OpenMPI
- Built -openmpi sub-packages
- Built a common sub-package
- Performed serial and parallel tests

* Sat Aug 30 2014 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.11.9-1
- Update to latest upstream release (#1131008)

* Sat Aug 16 2014 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.11.8-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_21_22_Mass_Rebuild

* Wed Jul 9 2014 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.11.8-1
- Add missing requires to MUMPS-devel in the devel package
- Do not explicitly link to openmpi
- Update to version that corrects #1117974
- Update to latest upstream release (#1089927)

* Fri Jun 13 2014 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.11.0-6
- Adapt to newer scalapack correcting rawhide FTBFS (#1106072)

* Sat Jun 07 2014 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.11.0-5
- Rebuilt for https://fedoraproject.org/wiki/Fedora_21_Mass_Rebuild

* Mon Apr  7 2014 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.11.0-4
- Correct files conflict (#1084893).

* Wed Aug  7 2013 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.11.0-3
- Switch to unversioned docdir.
- Correct rawhide FTBFS (#992075).

* Sat Aug 03 2013 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.11.0-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_20_Mass_Rebuild

* Sat May 11 2013 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.11.0-1
- Update to latest upstream release.
- Adjust patches and build for sources now in toplevel directory.

* Thu May 9 2013 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.10.4-1
- Update to latest upstream release.
- Switch to the new upstream tarballs without bundled dependencies.

* Sat Mar  2 2013 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.10.3-2
- Remove ThirdParty directory.
- Make use of the MUMPS solver (#913152).
- Split html documentation in a doc package.

* Mon Jan 14 2013 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.10.3-1
- Update to latest upstream release.

* Sat Jan 12 2013 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.10.2-3
- Rename repackaged tarball.

* Sun Nov 18 2012 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.10.2-2
- Rename package to coin-or-Ipopt.
- Do not package Thirdy party data or data without clean license.

* Sat Sep 29 2012 pcpa <paulo.cesar.pereira.de.andrade@gmail.com> - 3.10.2-1
- Initial coinor-Ipopt spec.

